
10  ' Future enhancements that may happen
20  ' Allow 8 bit writes (and have matching initialisation code)
30  ' Describe extra wiring needed to read data from the LCD
40  '    Set up Read to work with 4 bits (or 8)
50  '    Read the BUSY status and address
60  '    Add timers that check how long control and display actions take
70  '    Save 4-bit / 8-bit flag to file, like LCD size
80  '
90  ' Edit CGRAM characters and display result
100 ' Status bar showing status of each flag
110 ' Memory map showing contents of LCD memory(and memory that cannot be read from)
120 ' Demo of CGRAM read, showing how 2 different addresses return the same character
130 ' Demo showing how characters written to some bits of DD memory cannot be read
140 '
190 ' ----------------------------------------------------------------------------------
200 ' 				' You shouldn't have to change the next 4 values
210  _C_L1_Start = 0
220  _C_L2_Start = 64
230  _C_L3_Start = 20
240  _C_L4_Start = 84
250 '
260 DIM _Offset(4)
270 _Offset(1) = _C_L1_Start	' load all the offsets into an array
280 _Offset(2) = _C_L2_Start
290 _Offset(3) = _C_L3_Start
300 _Offset(4) = _C_L4_Start
310 '
320 DIM _Text$(4)               ' An array to hold the 4 display lines
330 '
340 ' Change these values to change the initial display message.
350 ' We start with lines 1 and 2 having a 16 character length.
360 ' The code will pad the messages with spaces on the right
370 ' if the line length is longer than 16.
380 '
390 _Text$(1) = "Hello World, I'm"	' load initial display values for the screen
400 _Text$(2) = "a Maximite LCD  "
410 _Text$(3) = "communicating with a"	' if you have a 2 line display, there's no harm leaving these extra lines here
420 _Text$(4) = "4 line HD44780 LCD. "
430 '
440  ' Define the pin names
450  RS = 11
460  Enable = 12
470  D4 = 13  ' don't use until we figure out why the display gets corrupted when we say, for example, pin(D4) =
480  D5 = 14
480  D6 = 15
490  D7 = 16
500 '
510 _FullDisplay = 1                    '  This variable controls whether full or short descriptions are shown.
520 '
530 _LCD_Line = 1
540 _LCD_Col = 1                        ' defaults in case the user doesn't set a value
550 '
1000 ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
1010 ' V1.1 LCD Explorer for standard 16 X 2 and 20 X 4 LCDs
1020 '               James Deakins July 2011,
1030 ' based on the original 16 X 2 code by
1040 '               Geoff Graham, March 2011
1050 '
1060 '  For example:  futurlec.com      LCD16X2
1070 '               altronics.com.au   Z7001
1080 '               jaycar.com.au      QP5512
1090 '               eBay               HD44780 based 20x4, has 2004A on crt board
1100 '
1110 ' To use:
1120 '  - Set up your text messages and number of lines / characters for your LCD
1135 '   - execute GOSUB 10200 to set up variables for the program
1130 '	 - execute GOSUB 29000 to initialise the LCD display
1140 '   - execute GOSUB 11000 to display the Explorer menu
1145 '
1150 '   - Expect to see very different results if you're comparing a 2 and 4 line LCD.
1160 '
1170 '   - Cycle between the settings and display menus, choosing a control setting
1180 '     and seeing what the effect is
1190 '
1200 '   - Select H on the Settings menu to toggle descriptions with the options
1210 '
1220 ' Make sure you have the right pinouts (see below).  There's a new
1230 ' schematic on Geoff's site, but if you have the old one, USE THIS
1240 ' PINOUT INFORMATION INSTEAD OF THE PINOUTS ON THE SCHEMATIC!!
1250 ' **************************************************************
1260 ' Maximite pin 11 is RS, pin 12 is EN. R/W is grounded.
1270 ' >>>pins 13 to 16 are D4 to D7.<<< May be different to schematic!
1280 ' **************************************************************
1290 ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
1300 '
10000 ' Main Code
10010 '
10020 GOSUB 10200               'Make sure user has supplied lines and columns
10030 GOSUB 29000		'Initialise the LCD
10040 GOSUB 11000		'Display the LCD Settings Menu
10050 END
10060 '
10070 ' End of Main Code
10080 '
10100 ' Subroutines
10110 ' -----------
10120 '
10200 ' Check if the program has previously stored values for LCD Lines and Columns
10210 '
10220
10230 OPTION ERROR CONTINUE        ' Don't abort if the open fails - we'll handle it
10240 OPEN "LCD_EXP.DAT" FOR INPUT AS #1
10250 IF MM.ERRNO = 6 then
10260   Input "Please enter the number of lines and columns for your LCD (eg 2,16) "; _LCD_Lines, _LCD_LineLength
10270   OPEN "LCD_EXP.DAT" FOR OUTPUT AS #1
10280   WRITE #1, _LCD_Lines, _LCD_LineLength
10290   Close #1
10300 ELSE
10310    INPUT #1, _LCD_Lines, _LCD_LineLength
10320    CLOSE #1
10330 ENDIF
10340 OPTION ERROR ABORT
10350 '
10360 ' now fill out the demo text with spaces on the right (otherwise
10370 ' strange characters are printed)
10380 For I = 1 to _LCD_Lines
10390   _Spaces = _LCD_LineLength - LEN(_Text$(I))
10400   If _Spaces > 0 THEN
10410      _Text$(I) = _Text$(I) + Space$(_Spaces)
10420   ENDIF
10430 NEXT I
10980 RETURN
10990 '
11000 ' Display the menu
11010 '
11020 '
11030 CLS
11040 IF _FullDisplay = 0 then
11050    GOSUB 11300  ' terse display (aka Expert Mode)
11060 ELSE
11070    GOSUB 11600  ' full display (aka Help Display)
11080 ENDIF
11090 '
11100 Print
11110 Print "Enter a number above, or D to see how the Display is affected by the settings"
11115 Print "Enter Q to Quit, C to Change LCD size, or H to toggle the Help display"
11120 INPUT "Please enter your selection: ", Sel$  : Sel$ = UCASE$(Sel$)
11130 IF Sel$ >= "1" AND Sel$ <= "9" THEN
11140    Sel = Val(Sel$)
11150    on Sel GOSUB 12000, 12100, 12200, 12300, 12400, 12500, 12600, 12700, 12800
11160 ELSEIF Sel$ = "Q" THEN
11170    RETURN
11180 ELSEIF Sel$ = "H" THEN
11190    _FullDisplay = NOT _FullDisplay  ' toggle the value
11200 ELSEIF Sel$ = "C" THEN
11210    GOSUB 22100
11220 ELSEIF LEFT$(Sel$,1) = "D" THEN
11230    GOSUB 14100
11240 ELSE
11250   Print "Sorry, that isn't a valid choice."
11260   pause 3000
11270 ENDIF
11280 GOTO 11000
11290
11300 ' Terse Display (Expert Mode)
11350 Print
11370 Print "[1]  Clear Display            "
11390 Print "[2]  Return Home              "
11420 Print " 3   Entry Mode SET           "
11440 Print " 4   Display ON/OFF Control   "
11460 Print " 5   Cursor or Display Shift  "
11480 Print " 6   Function Set             "
11500 Print " 7   Set CG RAM Address       "
11510 Print " 8   Set DD RAM Address (L,C) "
11520 Print " 9   Set DD RAM Address (Addr)"
11570 RETURN
11600 ' Long Display
11610 Print "This menu lets you try all the LCD setup options."
11620 Print "Some activate immediately.  Some only show results when you display some text."
11630 Print "The ones that activate immediately are in []s, the others prompt for values."
11640 Print "Please enter the number next to the option you want to try."
11650 Print
11670 Print "[1]  Clear Display  (Write '20H' (space) to DDRAM and set DDRAM address"
11680 Print "                      to '00H' from Address Counter)"
11690 Print "[2]  Return Home    (Sets DDRAM address to '00H' from Address Counter and"
11700 Print "                      return cursor to its original position if shifted."
11710 Print "                      DDRAM contents are not changed)"
11720 Print " 3   Entry Mode SET (Assign cursor moving direction and enable shifting of"
11730 Print "                      the entire display)"
11740 Print " 4   Display ON/OFF Control  (Set control bits for Display, Cursor and"
11750 Print "                               Blinking Cursor)"
11760 Print " 5   Cursor or Display Shift (Set cursor moving and display shift control bit"
11770 Print "                              and direction without changing DDRAM data)"
11780 Print " 6   Function Set   (Sets Interface Data Length (4 or 8 bits) number of display "
11790 Print "                     lines (1 or 2) and display font type (5x11 or 5x8 dots))"
11800 Print " 7   Set CG RAM Address (Sets Character Generator RAM addr in address counter)"
11810 Print " 8   Set DD RAM Address (Sets DD RAM address in address counter using line, col)"
11820 Print " 9   Set DD RAM Address (Sets DD RAM address in address counter using Address)"

11870 RETURN
12000 ' 1 selected - Clear Display
12001 ' --------------------------
12002 '
12010 _LCD_byte = &B00000001 : GOSUB 30000
12090 RETURN

12100 ' 2 selected - Return Home
12101 ' ------------------------
12102 '
12110 _LCD_byte = &B00000010 : GOSUB 30000
12190 RETURN

12200 ' 3 Selected - Character Entry Mode
12201 ' ---------------------------------
12202 '
12210 Input "Assign Cursor Moving Direction (L=Left, R=Right) ", Answer$
12220 CurDir$ = UCASE$(Answer$)
12230 Input "Enable shifting of entire display (Y,N) ", Answer$
12240 DispShift$ = UCASE$(Answer$)
12250 _LCD_byte = &B00000100
12260 if CurDir$ = "L" then _LCD_byte =    _LCD_byte OR &B00000010
12270 if DispShift$ = "Y" then _LCD_byte = _LCD_byte OR &B00000001
12280 GOSUB 30000
12290 RETURN

12300 ' 4 selected - Display ON/OFF Control
12301 ' -----------------------------------
12302 '
12305 Input "Set Display ON (Y,N) ";Answer$
12310 DispOn$ = UCASE$(Answer$)
12315 Input "Set Cursor ON (Y,N) "; Answer$
12320 CurOn$ = UCASE$(Answer$)
12325 Input "Set Cursor Blink ON (Y,N)"; Answer$
12330 CurBlinkOn$ = UCASE$(Answer$)
12335 _LCD_byte = &B00001000
12340 If DispOn$ = "Y" then _LCD_byte =     _LCD_byte OR &B00000100
12345 If CurOn$ = "Y" then _LCD_byte =      _LCD_byte OR &B00000010
12350 If CurBlinkOn$ = "Y" then _LCD_byte = _LCD_byte OR &B00000001
12380 GOSUB 30000
12390 RETURN

12400 ' 5 selected - Cursor or Display Shift
12401 ' ------------------------------------
12402 '
12410 Input "Set Cursor move or Display shift (C,D) "; Answer$
12420 CorD$ = UCASE$(Answer$)
12430 Input "Set Left shift or Right shift (L,R) "; Answer$
12440 LorR$ = UCASE$(Answer$)
12450 _LCD_byte = &B00010000
12460 If CorD$ = "D" then _LCD_byte = _LCD_byte OR &B00001000
12470 If LorR$ = "R" then _LCD_byte = _LCD_byte OR &B00000100
12480 _Shift = _LCD_Byte
12485  ' GOSUB 30000    ' we don't need to call it, we use it for each character shifted
12490 RETURN

12500 ' 6 selected - Function Set
12501 ' -------------------------
12502 '
12505 Input "Set Data Length (how many bits) (4,8) ";DataLen$
12510 Input "Set Number of Display Lines [answer 2 for 4 line displays] (1,2) ";DispLines$
12515 Input "Set Display Font [B=Big=5x11, S=Small=5x8] (B,S) ";Answer$
12520 DispFont$ = UCASE$(Answer$)
12530 _LCD_byte = &B00100000
12540 If DataLen$ = "8" then _LCD_byte =   _LCD_byte OR &B00010000
12550 If DispLines$ = "2" then _LCD_byte = _LCD_byte OR &B00001000
12555 If DispFont$ = "B" then _LCD_byte =  _LCD_byte OR &B00000100
12560 If DispLines$ <> "1" AND DispFont$ = "B" THEN
12570      Input "Specification says Big font only works with Single Line displays.  Continue ? (Y,N): ",Answer$ : Answer$ = UCASE$(Answer$)
12575      If Answer$ <> "Y" THEN GOTO 12500
12577 ENDIF
12578 If DispFont$ = "B" then 
12579      PRINT "Normal text doesn't look any different when using Big fonts.
12580      PRINT "Try Display Option 4 with special character EA to see the effect."
12582 ENDIF
12585 GOSUB 30000
12590 RETURN

12600 ' 7 selected Set CGRAM Address
12601 ' ----------------------------
12602 '
12610 Input "Please enter the address for CGRAM (0 to 63) "; Addr
12620 _LCD_byte = &B01000000
12630 _LCD_byte = _LCD_byte OR Addr
12640 GOSUB 30000
12690 RETURN

12700 ' 8 selected - Set DDRAM address  (using Line and Column)
12710 ' -------------------------------------------------------
12720 '
12730 Print "Lines and columns start at 1 (there is no Line 0 or Column 0)."
12740 Input "Please enter the line and column (Line, Col) "; _LCD_Line, _LCD_Col
12750 GOSUB 30200   ' GOTO the Line and Col
12760 RETURN

12800 ' 9 selected - Set DDRAM address
12810 ' ------------------------------
12820 '
12830 Input "Please enter the address for DDRAM (0 to 127) "; Addr
12840 IF (Addr >=40 AND Addr <= 63) OR (Addr >= 104 AND Addr <= 127) THEN
12850    Print "That address cannot be used, but we'll try it anyway"
12860 ENDIF
12870 _LCD_byte = &B10000000
12880 _LCD_byte = _LCD_byte OR Addr
12890 GOSUB 30000
12900 RETURN

14100 ' D selected - Show the DISPLAY menu
14110 ' ----------------------------------
14120 '
14130 CLS
14140 Temp$ = Right$(Sel$,1)                    ' allow user to pre-select - ie D3
14150 IF Temp$ >= "1" AND Temp$ <= "9" THEN
14160    Sel = Val(Temp$)
14170    GOTO 15070
14180 ENDIF
14190 '
14200 Print "Make sure that you have used Option 8 to set the DD RAM address"
14210 Print "to point to the line that you want the message displayed on"
14220 Print
14230 Print " 1  Display the standard 1, 2 or 4 line greeting message"
14240 Print " 2  Display 26 characters, ready to test scrolling (option 5)"
14250 Print " 3  Display standard characters at the location you previously selected"
14260 Print " 4  Print a non-standard character at the loc'n you previously selected"
14270 Print " 5  Scroll the current screen contents
14280 Print " 6  Use custom code to scroll a long message on the line you prev selected"
14290 Print " 7  Read the 'Busy' flag and address (The Busy flag tells you if the panel"
14300 Print "           is still doing internal operations.  Also allows AC to be read)"
14310 Print " 8  Write DATA to RAM (Writes data into internal RAM (DDRAM / CGRAM)"
14320 Print " 9  Read DATA from RAM (Reads data from internal RAM (DDRAM / CGRAM)"
14330 Print " 10  Run your code.  Write code from L 40000 in a seperate file, then MERGE it."


15000 INPUT "Please enter your selection: ", Sel
15010 If Sel < 0 OR Sel > 10 then
15020   Print "Sorry, that isn't a valid choice."
15030   pause 3000
15040   goto 13100
15050 endif
15060 '
15070 ON Sel GOSUB 20000, 20150, 20400, 20500, 20600, 20900, 21100, 21400, 21700, 40000
15080 RETURN


20000 ' 1. Display the default messages for the lines (defined at beginning of the code)
20010 ' -----------------------------------------------------------------------------
20020 ' This ignores whatever DDRAM location you have set up with the Control menu.
20030 ' It relies on the values for _C_Lines and _C_LineLength being correct.
20040 '
20050 for _Line_out = 1 to _LCD_Lines
20060	_LCD_byte = 128 + _Offset(_Line_out) : GOSUB 30000	' select the output line on the LCD
20070	for _LCD = 1 to _LCD_LineLength
20080		_LCD_byte = ASC(MID$(_Text$(_Line_out), _LCD, 1))
20090		pin(RS) = 1 : GOSUB 30000			' send the character
20100	next _LCD
20110 next _Line_out
20120 '
20130 RETURN

20150 ' 2. Display a 26 character message, ready to test scrolling (option 5)
20160 ' ---------------------------------------------------------------------
20170 '
20180 ' Display a 26 char message at the chosen location
20190 '  (noting that on a 16 / 20 character display, we only see the first 16 / 20 characters)
20200 ' ----------------------------------------------------------------------------------------------------
20210 TestMsg$ = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
20220 for _LCD = 1 to 26
20230 	_LCD_byte = ASC(MID$(TestMsg$, _LCD, 1))
20240	pin(RS) = 1 : GOSUB 30000				' Send the character
20250 next _LCD
20260 '
20270 ' the user should now see the first 16 / 20 characters of our test message
20280 '
20290 Print "You should now see 16 or 20 characters, depending on your screen.  Try option 5 to scroll it."
20300 RETURN

20400 ' 3. Print user entered characters at the selected location
20410 '----------------------------------------------------------
20420 Print "Notice that the character is selected from the table in the LED module"
20430 Print "so it may be different from the character you type in.  For example, try \"
20435 Print
20440 Print "Enter multiple (or even just one) character/s.  They will be displayed"
20445 Print "at the location you selected.  Enter an X to exit. ";
20450 Key$ = Inkey$ : IF Key$ = "" THEN GOTO 20450
20460 IF UCASE$(Key$) = "X" THEN
20465      RETURN
20470 ELSE
20475     _LCD_byte = ASC(Key$) : pin(RS) = 1 : gosub 30000		' Write the character
20480 ENDIF
20490 GOTO 20450

20500 ' 4. Print a non-standard character from the character set held on the LCD
20510 ' ------------------------------------------------------------------------
20520 Input "Please input the hex value of the non-standard character you want to print (ie F4) "; _HexVal$
20530 _ExtVal$ = "&H" + _HexVal$
20540 GOSUB 31000
20550 _DecVal = _ExtVal   ' a bit of a waste - I could just use _ExtVal in the line below, but good for debugging.
20560 _LCD_byte = _DecVal : pin(RS) = 1: GOSUB 30000     ' print the non standard character
20590 RETURN

20600 ' 5. Shift the message so the balance of the message can be read
20610 ' --------------------------------------------------------------
20620 ' Note:  we use the value stored in _Shift, which was set in Control Option 5 - Set Cursor
20630 ' or Display shift. That way, we make sure the shifting goes in the correct direction.
20640 '
20650 GOSUB 30200		' for variety, we use the LCD LOCATE subroutine
20660 Print "Press C to auto complete the 40 char scroll, X to exit and leave scrolling incomplete"
20670 Print "or any other key to scroll one position.  You can use these keys at any time. ";
20680 _ScrCtr = 1
20690 Key$ = Inkey$ : Key$ = UCASE$(Key$)
20700 If Key$ = "" then
20710     GOTO 20690
20720 ELSEIF Key$ = "X" then
20730     RETURN
20740 ELSEIF Key$ = "C" then
20750    for _LCD = _ScrCtr to 40
20760         _LCD_byte = _Shift : GOSUB 30000
20770         pause 50
20780     next _LCD
20790     RETURN
20800 ELSE     ' move one position
20810    _LCD_byte = _Shift : GOSUB 30000
20820    _ScrCtr = (_ScrCtr + 1) MOD 40
20830 ENDIF
20840 GOTO 20690
20850 '
20860 ' Note that this shifts both lines, and you have to shift 40 times to get back where you started from
20870 ' ---------------------------------------------------------------------------------------------------
20880 ' Also notice how the text may change to different lines as the message is scrolled.
20890 '

20900 ' 6. use custom code to scroll a message on a given line
20910 ' ------------------------------------------------------
20920 Print "You MUST set the location by Line and Column first (not a single DDRAM address) !"
20930 ' because we will use the values _LCD_Line and _LCD_Col
20940 ' and yes, it will fail if people put in a short message.  TODO: Fix this.
20950 Print
20960 Line Input "Please type in a long message: "; LongMessage$
20970 MsgLen = Len(LongMessage$)
20980 DispLen = _LCD_LineLength - _LCD_Col + 1
20990 For Mask_Offset = 0 to MsgLen - DispLen   		' keep track of where we're starting the display window
21000   GOSUB 30200                                             ' GOTO the previously selected line and column
21010 	for _LCD = 1 to DispLen
21020		_LCD_byte = ASC(MID$(LongMessage$, Mask_Offset + _LCD, 1))
21030		pin(RS) = 1 : GOSUB 30000
21040	next _LCD
21050   if Mask_offset = 0 then pause 2000 else pause 800	' so screen has time to refresh, and user can read it
21060 next Mask_Offset
21070 RETURN

21100 ' Option 7 - Read 'Busy' flag and address - the Address could be really useful if the user
21110 '                 uses direction keys to move cursor and select an option.
21120 PRINT "Not implemented yet"
21130 PAUSE 3000
21140 RETURN

21400 ' Option 8 -  Write DATA to RAM (only really useful for CGRAM, because DDRAM done elsewhere)
21410 PRINT "If this looks weird, you had to set the CGRAM address first"
21420 PRINT "Follow this step by selecting option Display 9 (D9)
21430 _LCD_byte = &H0E : Pin(RS) = 1 : GOSUB 30000 ' Load byte 1 to CGRAM
21440 _LCD_byte = &H11 : Pin(RS) = 1 : GOSUB 30000 ' Load byte 2 to CGRAM
21450 _LCD_byte = &H0E : Pin(RS) = 1 : GOSUB 30000 ' Load byte 3 to CGRAM
21460 _LCD_byte = &H04 : Pin(RS) = 1 : GOSUB 30000 ' Load byte 4 to CGRAM
21470 _LCD_byte = &H1F : Pin(RS) = 1 : GOSUB 30000 ' Load byte 5 to CGRAM
21480 _LCD_byte = &H04 : Pin(RS) = 1 : GOSUB 30000 ' Load byte 6 to CGRAM
21490 _LCD_byte = &H0A : Pin(RS) = 1 : GOSUB 30000 ' Load byte 7 to CGRAM
21500 _LCD_byte = &H11 : Pin(RS) = 1 : GOSUB 30000 ' Load byte 8 to CGRAM
21510 RETURN


21700 ' Option 9 - Read DATA from RAM
21710 Input "Display all 8 characters from CG RAM? ";Answer$ : Answer$ = UCASE$(Answer$)
21720 if Answer$ = "Y" THEN
21730    FOR _LCD_Addr = 0 to 7
21740        _LCD_byte = _LCD_Addr : pin(RS) = 1: GOSUB 30000     ' print the next user defined character in CG RAM
21750    NEXT _LCD_Addr
21760 ENDIF
21780 RETURN

22100 ' Let the user change the number of lines and columns on the LCD screen
22110 '
22120 Input "Please enter the number of lines and columns for your LCD (eg 2,16) "; _LCD_Lines, _LCD_LineLength
22130 OPEN "LCD_EXP.DAT" FOR OUTPUT AS #1
22140 WRITE #1, _LCD_Lines, _LCD_LineLength
22150 Close #1
22160 RETURN

29000 ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
29010 ' Initialise the LCD
29020 '
29030 FOR i = 11 TO 16 : SETPIN i, 9 : NEXT i	   ' set the pins to Open collector digital output to 5V
29040 _LCD_byte = &B0011 : GOSUB 30100             ' reset
29050 _LCD_byte = &B0011 : GOSUB 30100             ' reset
29060 _LCD_byte = &B0011 : GOSUB 30100             ' reset
29070 _LCD_byte = &B0010 : GOSUB 30100             ' 4 bit mode
29080 _LCD_byte = &B00101100 : GOSUB 30000         ' 4 bit, 2 lines (or 4 lines)
29090 _LCD_byte = &B00001100 : GOSUB 30000         ' display on, no cursor
29100 _LCD_byte = &B00000110 : GOSUB 30000         ' increment on write
29110 _LCD_byte = &B00000001 : GOSUB 30000         ' clear display
29120 pause 2
29130 RETURN
30000 ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
30010 ' Send a byte to the LCD
30020 ' the data to be sent is in _LCD_byte
30030 '
30040 PIN(13) = _LCD_byte AND &B00010000      ' output the 1st 4 bits
30050 PIN(14) = _LCD_byte AND &B00100000
30060 PIN(15) = _LCD_byte AND &B01000000
30070 PIN(16) = _LCD_byte AND &B10000000
30080 pin(Enable) = 1 : pin(Enable) = 0               ' tell lcd to accept data
30090 pause 2
30100 ' Entry point to send just 4 bits to the LCD
30110 PIN(13) = _LCD_byte AND &B00000001      ' output the 2nd 4 bits
30120 PIN(14) = _LCD_byte AND &B00000010
30130 PIN(15) = _LCD_byte AND &B00000100
30140 PIN(16) = _LCD_byte AND &B00001000
30150 pin(Enable) = 1 : pin(Enable) = 0 : pin(RS) = 0 ' tell lcd to accept data
30160 RETURN
30170 '
30200 ' Cursor LOCATE code
30210 '
30220 ' Move cursor to a location on the LCD screen
30240 ' The line is sent in _LCD_Line, the column is sent in _LCD_Col
30250 _Start = _Offset(_LCD_Line) + _LCD_Col -1
30260 IF (_Start >=40 AND _Start <= 63) OR (_Start >= 104 AND _Start <= 127) THEN
30270    Print "That address cannot be used, but we'll try it anyway."
30280 ENDIF
30290 _LCD_byte = 128 + _Start : GOSUB 30000	' move to the requested location
30300 RETURN

31000 ' Replacement code for the VAL() Function.  It currently only accepts a string that represents decimal values.
31010 ' VAL() does not convert Binary, Octal or Hexidecimal strings to Decimal.
31020 ' Usage: Store the string to be converted in a variable called _ExtVal$ before calling this subroutine.
31025 '
31030 '        The decimal result will be returned in a variable called ExtVal.
31040 '
31050 IF _ExtVal$ = "" THEN
31060    ERROR "Empty string passed to Extended VAL() subroutine"
31070 ENDIF
31080 _Val$ = UCASE$(_ExtVal$)      ' We will work with upper case
31090 _Type$ = LEFT$(_Val$,2)
31100 _ValLen = LEN(_Val$)
31110 IF _Type$ = "&H" THEN         ' Hex value
31120    GOSUB 21300
31130 ELSEIF _Type$ = "&O" THEN
31130    GOSUB 21600                ' Octal value
31150 ELSEIF _Type$ = "&B" THEN
31160    GOSUB 21900
31170 ELSE
31180    _ExtVal = VAL(_ExtVal$)    ' Must be decimal
31190 ENDIF
31200 RETURN

31300 ' A Hex value was passed to the Extended Val subroutine
31310 _ExtVal = 0
31320 _Power = 1
31330 _ValLen = _ValLen - 2                ' &H take up 2 character positions
31340 _Val$ = RIGHT$(_Val$, _ValLen)       ' strip off the &H
31350 FOR _X = _ValLen to 1 STEP -1
31360    _Char$ = MID$(_Val$, _X, 1)       ' evaluate characters right to left
31370    IF _Char$ >= "0" AND _Char$ <= "9" THEN
31380       _CharVal = VAL(_Char$)
31390    ELSEIF _Char$ >= "A" AND _Char$ <= "F" THEN
31400       _CharVal = ASC(_Char$) - 55
31410    ELSE
31420       PRINT "Invalid character "; _Char$; " found in Hex Value passed to Extended Val subroutine"
31430    ENDIF
31440    _ExtVal = _ExtVal + _CharVal * _Power
31450    _Power = _Power * 16
31460 NEXT _X
31470 RETURN

31600 ' An Octal value was passed to the Extended Val subroutine
31610 _ExtVal = 0
31620 _Power = 1
31630 _ValLen = _ValLen - 2                ' &O take up 2 character positions
31640 _Val$ = RIGHT$(_Val$, _ValLen)       ' strip off the &O
31650 FOR _X = _ValLen to 1 STEP -1
31660    _Char$ = MID$(_Val$, _X, 1)       ' evaluate characters right to left
31670    IF _Char$ >= "0" AND _Char$ <= "8" THEN
31680       _CharVal = VAL(_Char$)
31690    ELSE
31700       PRINT "Invalid character "; _Char$; " found in Octal Value passed to Extended Val subroutine"
31710    ENDIF
31720    _ExtVal = _ExtVal + _CharVal * _Power
31730    _Power = _Power * 8
31740 NEXT _X
31750 RETURN

31900 ' A Binary value was passed to the Extended Val subroutine
31910 _ExtVal = 0
31920 _Power = 1
31930 _ValLen = _ValLen - 2                ' &B take up 2 character positions
31940 _Val$ = RIGHT$(_Val$, _ValLen)       ' strip off the &B
31950 FOR _X = _ValLen to 1 STEP -1
31960    _Char$ = MID$(_Val$, _X, 1)       ' evaluate characters right to left
31970    IF _Char$ >= "0" AND _Char$ <= "1" THEN
31980       _CharVal = VAL(_Char$)
31990    ELSE
32000       PRINT "Invalid character "; _Char$; " found in Binary Value passed to Extended Val subroutine"
32010    ENDIF
32020    _ExtVal = _ExtVal + _CharVal * _Power
32030    _Power = _Power * 2
32040 NEXT _X
32050 RETURN

40000 ' This is a placeholder for the users custom code
40010 ' Just write code with line numbers starting from 40000
40020 ' and MERGE it in with this program.  Then select the option to run custom code.
40030 PRINT "This would be replaced by your custom code"
40040 RETURN
